from ipywidgets import Output

from chosco.annotation.domain.annotation import Annotation
from chosco.annotation.domain.annotation_repository import AnnotationRepository
from chosco.media.retriever.domain.media_retriever import MediaRetriever
from chosco.view.interactor.domain.view_interactor import ViewInteractor


class Annotator:
    def __init__(
        self,
        username: str,
        retriever: MediaRetriever,
        repository: AnnotationRepository,
        view_interactor: ViewInteractor,
    ):
        self.username = username
        self.retriever = retriever
        self.repository = repository
        self.view_interactor = view_interactor
        self.current_image_id = None

        def previous_item():
            image_id, image = self.retriever.previous_image()
            self.current_image_id = image_id
            progress = self.retriever.progress()
            progress_max = self.retriever.total_images
            progress_value = self.retriever.current_index + 1
            self.view_interactor.update_header_box(
                image_id, image, progress, progress_max, progress_value
            )

        def next_item():
            image_id, image = self.retriever.next_image()
            self.current_image_id = image_id
            progress = self.retriever.progress()
            progress_max = self.retriever.total_images
            progress_value = self.retriever.current_index + 1
            self.view_interactor.update_header_box(
                image_id, image, progress, progress_max, progress_value
            )

        def confirm_annotation(create_metadata):
            metadata = create_metadata()
            annotation = Annotation(self.current_image_id, metadata)
            self.repository.save(annotation)
            self.repository.show()

        self.view_interactor.set_callbacks(
            {
                "next_item": next_item,
                "previous_item": previous_item,
                "confirm_annotation": confirm_annotation,
            }
        )

    def execute(self, output: Output):
        self.view_interactor.execute(output)



import json

from chosco.annotation.domain.annotation import Annotation
from chosco.annotation.domain.annotation_repository import AnnotationRepository


class InMemoryAnnotationRepository(AnnotationRepository):
    def __init__(self):
        self.items = {}

    def save(self, annotation: Annotation):
        self.items[annotation.id] = annotation.metadata

    def show(self):
        print(json.dumps(self.items, indent=4))



from abc import ABC, abstractmethod
from typing import List, Tuple, Any


class MediaRetriever(ABC):
    def __init__(self, folder: str):
        self.folder = folder

    @abstractmethod
    def previous_image(self, not_allowed_images: List[str] = None) -> Tuple[str, Any]:
        raise NotImplementedError

    @abstractmethod
    def next_image(self, not_allowed_images: List[str] = None) -> Tuple[str, Any]:
        raise NotImplementedError

    @abstractmethod
    def progress(self) -> str:
        raise NotImplementedError



from chosco.annotation.application.annotator import Annotator
from chosco.annotation.domain.annotation import Annotation
from chosco.annotation.domain.annotation_repository import AnnotationRepository
from chosco.annotation.infrastructure.inmemory_annotation_repository import (
    InMemoryAnnotationRepository,
)
from chosco.media.retriever.domain.media_retriever import MediaRetriever
from chosco.media.retriever.infrastructure.web_media_retriever import WebMediaRetriever
from chosco.view.interactor.document_annotation_view_interactor import (
    DocumentAnnotationViewInteractor,
)



from typing import List, Tuple

from IPython.core.display import Image
from chosco.media.retriever.domain.media_retriever import MediaRetriever


class WebMediaRetriever(MediaRetriever):
    def __init__(self, max_width: int = 1200):
        self.url = [
            "https://participa.podemos.info/assets/user_verifications/dni-sample1-489cd19dc5c69fc6b386e4da711536cc5579d903f9d80ab84af5d90f56b687fa.png",
            "https://cadenaser00.epimg.net/ser/imagenes/2020/02/06/sociedad/1581019912_200613_1581093640_noticia_normal.jpg",
            "https://upload.wikimedia.org/wikipedia/commons/thumb/7/70/Deutscher_Personalausweis_im_ab_2021_vorgesehenen_Design%2C_Bundesregierung_der_Bundesrepublik_Deutschland%2C_Entwurf_eines_Gesetzes_zur_St%C3%A4rkung_der_Sicherheit_im_Pass-%2C_Ausweis-_und_ausl%C3%A4nderrechtlichen_Dokumentenwesen.jpeg/542px-thumbnail.jpeg",
            "https://www.journalducameroun.com/en/wp-content/uploads/2020/08/Digital-ID-1-780x440.png",
        ]
        self.current_index = None
        self.total_images = len(self.url)
        self.max_width = max_width

    def _increase_index(self):
        if self.current_index is None:
            self.current_index = 0
        else:
            if self.current_index < self.total_images - 1:
                self.current_index += 1

    def _decrease_index(self):
        if self.current_index is None:
            self.current_index = 0
        else:
            if self.current_index >= 1:
                self.current_index -= 1

    def previous_image(self, not_allowed_images: List[str] = None) -> Tuple[str, Image]:
        self._decrease_index()
        url = self.url[self.current_index]
        return url, Image(url, unconfined=True, width=self.max_width)

    def next_image(self, not_allowed_images: List[str] = None) -> Tuple[str, Image]:
        self._increase_index()
        url = self.url[self.current_index]
        return url, Image(url, unconfined=True, width=self.max_width)

    def progress(self) -> str:
        return f"{self.current_index + 1} of {self.total_images}"



import functools
from typing import Dict, Callable, List
from ipywidgets import (
    Button,
    IntProgress,
    Label,
    Layout,
    Box,
    Output,
    Image,
    Tab,
    CoreWidget,
    Accordion,
)
from IPython.core.display import display

from chosco.view.interactor.domain.view_interactor import ViewInteractor


class DocumentAnnotationViewInteractor(ViewInteractor):
    def __init__(
        self,
        required_widgets: List[CoreWidget],
        optional_widgets: List[CoreWidget] = None,
    ):
        self.required_widgets = required_widgets
        self.optional_widgets = optional_widgets
        self.row_layout = Layout(flex_flow="row", align_items="center", width="100%")
        self.column_layout = Layout(
            flex_flow="column", align_items="center", width="100%"
        )

    def set_callbacks(self, callbacks: Dict[str, Callable]):
        self.callbacks = callbacks

    def _create_headers_box(self, output):
        previous_button = Button(description="Previous")
        next_button = Button(description="Next")
        self.progress = IntProgress()
        self.item_id = Label("Click on Next to start the labeling")
        controller_box = Box(
            children=[previous_button, next_button, self.progress],
            layout=self.row_layout,
        )
        info_box = Box(children=[self.item_id], layout=self.row_layout)
        headers_box = Box(
            children=[controller_box, info_box], layout=self.column_layout
        )

        @output.capture(clear_output=True, wait=True)
        def update(b, callback):
            callback()

        previous_button.on_click(
            functools.partial(update, callback=self.callbacks.get("previous_item"))
        )
        next_button.on_click(
            functools.partial(update, callback=self.callbacks.get("next_item"))
        )

        return headers_box

    def _create_labeler_box(self, output):
        total_widgets = self.required_widgets

        required_widgets_box = Box(
            children=self.required_widgets, layout=self.column_layout
        )
        widgets_boxes = [required_widgets_box]

        if self.optional_widgets:
            optional_widgets_box = Box(
                children=self.optional_widgets, layout=self.column_layout
            )
            widgets_boxes.append(optional_widgets_box)

        widgets_tab = Tab(children=widgets_boxes)
        widgets_tab.set_title(0, "Required Annotations")
        if self.optional_widgets:
            widgets_tab.set_title(1, "Optional Annotations")

        confirm_button = Button(description="Confirm")
        create_labeler_box = Box(children=[widgets_tab, confirm_button])

        def create_metadata() -> Dict:
            metadata = {}
            for widget in total_widgets:
                if hasattr(widget, "description") and hasattr(widget, "value"):
                    metadata[widget.description] = widget.value

            return metadata

        @output.capture(clear_output=False, wait=True)
        def update(b, callback, create_metadata):
            callback(create_metadata)

        confirm_button.on_click(
            functools.partial(
                update,
                callback=self.callbacks.get("confirm_annotation"),
                create_metadata=create_metadata,
            )
        )
        return create_labeler_box

    def execute(self, output: Output):
        self.output = output
        headers_box = self._create_headers_box(output)
        labeler_box = self._create_labeler_box(output)

        main_layout = Layout(flex_flow="column", width="100%")
        main_box = Box(children=[labeler_box, output], layout=main_layout)

        labeler_accordion = Accordion([main_box])
        labeler_accordion.set_title(0, "Annotator")

        display(headers_box)
        display(labeler_accordion)

    def update_header_box(
        self,
        image_id: str,
        image: Image,
        progress: str,
        progress_max: int,
        progress_value: int,
    ):

        self.progress.description = progress
        self.progress.max = progress_max
        self.progress.value = progress_value
        self.item_id.value = image_id
        with self.output:
            display(image)
